==> glark-1.6.3.tar.gz: glark-1.6.3/glarkopt.rb <==
#!/usr/bin/ruby -w
# -*- ruby -*-

require './env'
require './log'
require './ansicolor'
require './glarkexpr'
require './glarkhelp'

class GlarkOptions

  attr_accessor :after
  attr_accessor :before
  attr_accessor :binary_files
  attr_accessor :count
  attr_accessor :directory
  attr_accessor :expr
  attr_accessor :exclude_matching_files
  attr_accessor :exit_status
  attr_accessor :explain
  attr_accessor :file_highlight
  attr_accessor :file_names_only
  attr_accessor :grep_output
  attr_accessor :highlight
  attr_accessor :infinite_distance
  attr_accessor :invert_match
  attr_accessor :nocase
  attr_accessor :num_matches
  attr_accessor :quiet
  attr_accessor :range_end
  attr_accessor :range_start
  attr_accessor :show_line_numbers
  attr_accessor :show_file_names
  attr_accessor :text_highlight
  attr_accessor :verbose
  attr_accessor :whole_lines
  attr_accessor :whole_words
  attr_accessor :write_null

  def initialize(package = "undef", version = "1.2.3.4")
    $options = self

    @after             = 0          # lines of context before the match
    @before            = 0          # lines of context after the match
    @binary_files      = "binary"   # 
    @count             = false      # just count the lines
    @directory         = "read"     # read, skip, or recurse, a la grep
    @expr              = nil    # the expression to be evaluated
    @exclude_matching_files = false # exclude files whose names match the expression
    @exit_status       = 1          # 0 == matches, 1 == no matches, 2 == error
    @explain           = false      # display a legible version of the expression
    @file_names_only   = false      # display only the file names
    @grep_output       = false      # emulate grep output
    @highlight         = true       # highlight matches (using ANSI codes)

    @infinite_distance = -1         # signifies no limit to the distance between
                                    # matches, i.e., anywhere within the entire file is valid.

    @invert_match      = false      # display non-matching lines
    @nocase            = false      # match case
    @num_matches       = nil        # the maximum number of matches to display per file
    @package           = package
    @quiet             = false      # minimize warnings
    @range_end         = nil        # range to stop searching; nil => the entire file
    @range_start       = nil        # range to begin searching; nil => the entire file
    @show_line_numbers = true       # display numbers of matching lines
    @show_file_names   = true       # show the names of matching files
    @verbose           = nil        # display debugging output
    @version           = version
    @whole_lines       = false      # true means patterns must match the entire line
    @whole_words       = false      # true means all patterns are '\b'ed front and back
    @write_null        = false      # in @file_names_only mode, write '\0' instead of '\n'

    # default highlighting
    @text_highlight    = ANSIColor::code("black on yellow")
    @file_highlight    = ANSIColor::code("reverse bold")
  end

  def run
    Log.log ""
    read_rcfile
    read_environment_variable

    # honor thy EMACS; go to grep mode
    set_grep_output if ENV["EMACS"]

    read_options
    validate
  end

  def read_rcfile
    Log.log ""
    # process the rc file
    if hd = Env.home_directory
      rc = hd + "/.glarkrc"
      Log.log "reading RC file: #{rc}"
      if File.exists?(rc)
        IO.readlines(rc).each do |line|
          line.sub!(/\s*#.*/, "")
          line.chomp!
          name, value = line.split(/\s*[=:]\s*/)
          next unless name && value

          case name
          when "after-context"
            @after = value.to_i
          when "before-context"
            @before = value.to_i
          when "binary-files"
            @binary_files = value
          when "context"
            @after = @before = value.to_i
          when "file-color"
            @file_highlight = make_highlight(name, value)
          when "grep"
            set_grep_output if to_boolean(value)
          when "highlight"
            @highlight = to_boolean(value)
          when "ignore-case"
            @nocase = to_boolean(value)
          when "quiet"
            @quiet = to_boolean(value)
          when "text-color"
            @text_highlight = make_highlight(name, value)
          when "verbose"
            @verbose = to_boolean(value) ? 1 : nil
            Log.verbose = @verbose
          when "verbosity"
            @verbose = value.to_i
            Log.verbose = @verbose
          end

        end
        
      end
    end
  end

  # creates a color for the given option, based on its value
  def make_highlight(opt, value)
    if value
      return ANSIColor::code(value)
    else
      $stderr.print "ERROR: " + opt + " requires a color\n"
      exit 2
    end
  end

  # returns whether the value matches a true value, such as "yes", "true", or "on".
  def to_boolean(value)
    [ "yes", "true", "on" ].include?(value.downcase)
  end

  def read_environment_variable
    # process the environment variable
    if e = ENV["GLARKOPTS"]
      options = e.split(/\s+/)
      while options.length > 0
        opt = options.shift
        process_option(opt, options)
      end
    end
  end

  # sets output a la grep
  def set_grep_output
    @highlight         = false
    @show_line_numbers = true
    @after             = 0
    @before            = 0
    @grep_output       = true
  end

  def read_options
    Log.log ""
    nargs = ARGV.size
    args = ARGV.dup

    @expr = nil

    while ARGV.length > 0
      arg = ARGV.shift
      break if @expr = process_option(arg, ARGV)
    end

    unless @expr
      # were any options processed?

      # A lone option of "-v" means version, if there was nothing else on the
      # command line. For grep compatibility, "-v" with an expression has to
      # mean an inverted match.

      if nargs == 1 && args[0] == "-v"
        show_version
      elsif nargs > 0
        $stderr.print "No expression provided.\n"
      end
      
      $stderr.print "Usage: glark [options] expression file...\n"
      $stderr.print "Try `glark --help' for more information.\n"
      exit 1
    end
  end

  def process_option(opt, args)
    Log.log "processing option #{opt}"
    case opt

      # after (context)
    when "-A"
      @after = args.shift.to_i
    when /^--after-context=(\d+)/
      @after = $1.to_i

      # before (context)
    when "-B"
      @before = args.shift.to_i
    when /^--before-context=(\d+)/
      @before = $1.to_i

      # after (range)
    when "-f", "--after"
      @range_start = args.shift

      # before (range)
    when "-b", "--before"
      @range_end = args.shift

      # range
    when "-R", "--range"
      @range_start, @range_end = args.shift, args.shift

      # context
    when "-C"
      nxt = args.shift
      # keep it if it is a number, else use the default
      if nxt =~ /^\d+/
        @before = @after = nxt.to_i
      else
        @before = @after = 2
        args.unshift(nxt)
      end
    when /^--context(=(\d+))?/
      @after = @before = if $2 then $2.to_i else 2 end
    when /^-(\d+)$/
      @after = @before = $1.to_i

      # highlighting
    when "-u", "--highlight", "-hl"
      @highlight = true
    when "-U", "--no-highlight", "-nohl"
      @highlight = false
      
      # version
    when "-V", "--version"
      show_version

      # verbose
    when "-_"
      @verbose = 1
      Log.verbose = @verbose
    when /^--verbos(?:e|ity)(?:=(\d+))?/
      @verbose = $1 ? $1.to_i : 1
      Log.log "setting verbose to #{@verbose}"
      Log.verbose = @verbose

    when "-v", "--invert-match"
      @invert_match = true
      @exit_status  = 0
    when "-i", "--ignore-case"
      @nocase = true

      # grep
    when "-g", "--grep"
      set_grep_output

      # help
    when "-?", "--help"
      GlarkHelp.new
      exit 0

      # regexp explanation
    when "-e", "--explain"
      @explain = true

      # line numbers
    when "-N", "--no-line-number"
      @show_line_numbers = false
    when "-n", "--line-number"
      @show_line_numbers = true

      # quiet
    when "-q", "-s", "--quiet", "--messages"
      @quiet = true
    when "-Q", "-S", "--no-quiet", "--no-messages"
      @quiet = false

    when "-m", "--match-limit"
      @num_matches = args.shift.to_i
      
      # whole words
    when "-w", "--word", "--word-regexp"
      @whole_words = true
    when "-W", "--no-word", "--no-word-regexp"
      @whole_words = false

      # whole lines
    when "-x", "--line-regexp"
      @whole_lines = true
    when "-X", "--no-line-regexp"
      @whole_lines = false

      # file names only
    when "-l", "--files-with-matches"
      @file_names_only = true
      @invert_match = false
    when  "-L", "--files-without-match"
      @file_names_only = true
      @invert_match = true

      # colors
    when "-T", "--text-color"
      @text_highlight = make_highlight(opt, args.shift)
    when "-F", "--file-color"
      @file_highlight = make_highlight(opt, args.shift)

    when "-c", "--count"
      @count = true

    when "-Z", "--null"
      @write_null = true

    when "-M", "--exclude-matching-filenames"
      @exclude_matching_files = true
      
    when "-d"
      @directory = args.shift
    when /^--directories=(\w+)/
      @directory = $1

    when "-r", "--recurse"
      @directory = "recurse"

    when "-o", "-a"
      ec = ExpressionCreator.new(opt, args)
      @expr = ec.expr
      return @expr               # we are done.

    when "-h", "--with-filename"
      @show_file_names = true
      
    when "-H", "--no-filename"
      @show_file_names = false
      
    when /^--binary-files?=\"?(\w+)\"?/
      @binary_files = $1
      Log.log "set binary_files to #{@binary_files}"
      
    when /^(\-\w)(.+)/
      opt, rest = $1, "-" + $2
      puts "opt, rest = #{opt}, #{rest}" if @verbose
      args.unshift(rest)
      puts "args = #{args}" if @verbose
      return process_option(opt, args)
      
      # the expression
    else
      Log.log "not an option: #{opt}"
      if args
        ec = ExpressionCreator.new(opt, args)
        @expr = ec.expr
        return @expr            # we are done.
      end
    end
    return nil                  # we're not done.
  end

  # check options for collisions/data validity
  def validate
    if @range_start && @range_end
      pctre = Regexp.new(/([\.\d]+)%/)
      smd = pctre.match(@range_start)
      emd = pctre.match(@range_end)
      if !smd == !emd
        if smd
          if smd[1].to_f > emd[1].to_f
            puts "ERROR: range start (#{smd}) follows range end (#{emd})"
            exit 2
          end
        elsif @range_start.to_i > @range_end.to_i
          puts "ERROR: range start (#{@range_start}) follows range end (#{@range_end})"
          exit 2
        end
      end
    end
  end

  def show_version
    print @package, ", version ", @version, "\n"
    print "Written by Jeff Pace (jpace@glark.org).\n"
    print "Released under the Lesser GNU Public License.\n"
    exit 0
  end
  
end


if __FILE__ == $0
  Log.verbose = true
  gopt = GlarkOptions.new(ARGV)
  gopt.run
  methods = GlarkOptions.instance_methods
  methods.sort.each do |meth|
    # call the accessor for every setter method
    if meth[-1, 1] == "="
      acc = meth[0 .. -2]
      m = gopt.method(acc)
      puts "#{acc}: #{m.call}"
    end
  end
end

==> glark-1.6.4.tar.gz: glark-1.6.4/glarkopt.rb <==
#!/usr/bin/ruby -w
# -*- ruby -*-

require './env'
require './log'
require './ansicolor'
require './glarkexpr'
require './glarkhelp'

class GlarkOptions

  attr_accessor :after
  attr_accessor :before
  attr_accessor :binary_files
  attr_accessor :count
  attr_accessor :directory
  attr_accessor :expr
  attr_accessor :exclude_matching_files
  attr_accessor :exit_status
  attr_accessor :explain
  attr_accessor :extract_matches
  attr_accessor :file_highlight
  attr_accessor :file_names_only
  attr_accessor :grep_output
  attr_accessor :highlight
  attr_accessor :infinite_distance
  attr_accessor :invert_match
  attr_accessor :nocase
  attr_accessor :num_matches
  attr_accessor :quiet
  attr_accessor :range_end
  attr_accessor :range_start
  attr_accessor :show_line_numbers
  attr_accessor :show_file_names
  attr_accessor :text_highlight
  attr_accessor :verbose
  attr_accessor :whole_lines
  attr_accessor :whole_words
  attr_accessor :write_null

  def initialize(package = "undef", version = "1.2.3.4")
    $options = self

    @after             = 0          # lines of context before the match
    @before            = 0          # lines of context after the match
    @binary_files      = "binary"   # 
    @count             = false      # just count the lines
    @directory         = "read"     # read, skip, or recurse, a la grep
    @expr              = nil    # the expression to be evaluated
    @exclude_matching_files = false # exclude files whose names match the expression
    @exit_status       = 1          # 0 == matches, 1 == no matches, 2 == error
    @explain           = false      # display a legible version of the expression
    @extract_matches   = false      # whether to show _only_ the part that matched
    @file_names_only   = false      # display only the file names
    @grep_output       = false      # emulate grep output
    @highlight         = true       # highlight matches (using ANSI codes)

    @infinite_distance = -1         # signifies no limit to the distance between
                                    # matches, i.e., anywhere within the entire file is valid.

    @invert_match      = false      # display non-matching lines
    @nocase            = false      # match case
    @num_matches       = nil        # the maximum number of matches to display per file
    @package           = package
    @quiet             = false      # minimize warnings
    @range_end         = nil        # range to stop searching; nil => the entire file
    @range_start       = nil        # range to begin searching; nil => the entire file
    @show_line_numbers = true       # display numbers of matching lines
    @show_file_names   = true       # show the names of matching files
    @verbose           = nil        # display debugging output
    @version           = version
    @whole_lines       = false      # true means patterns must match the entire line
    @whole_words       = false      # true means all patterns are '\b'ed front and back
    @write_null        = false      # in @file_names_only mode, write '\0' instead of '\n'

    # default highlighting
    @text_highlight    = ANSIColor::code("black on yellow")
    @file_highlight    = ANSIColor::code("reverse bold")
  end

  def run
    Log.log ""
    read_rcfile
    read_environment_variable

    # honor thy EMACS; go to grep mode
    set_grep_output if ENV["EMACS"]

    read_options
    validate
  end

  def read_rcfile
    Log.log ""
    # process the rc file
    if hd = Env.home_directory
      rc = hd + "/.glarkrc"
      Log.log "reading RC file: #{rc}"
      if File.exists?(rc)
        IO.readlines(rc).each do |line|
          line.sub!(/\s*#.*/, "")
          line.chomp!
          name, value = line.split(/\s*[=:]\s*/)
          next unless name && value

          case name
          when "after-context"
            @after = value.to_i
          when "before-context"
            @before = value.to_i
          when "binary-files"
            @binary_files = value
          when "context"
            @after = @before = value.to_i
          when "file-color"
            @file_highlight = make_highlight(name, value)
          when "grep"
            set_grep_output if to_boolean(value)
          when "highlight"
            @highlight = to_boolean(value)
          when "ignore-case"
            @nocase = to_boolean(value)
          when "quiet"
            @quiet = to_boolean(value)
          when "text-color"
            @text_highlight = make_highlight(name, value)
          when "verbose"
            @verbose = to_boolean(value) ? 1 : nil
            Log.verbose = @verbose
          when "verbosity"
            @verbose = value.to_i
            Log.verbose = @verbose
          end

        end
        
      end
    end
  end

  # creates a color for the given option, based on its value
  def make_highlight(opt, value)
    if value
      return ANSIColor::code(value)
    else
      $stderr.print "ERROR: " + opt + " requires a color\n"
      exit 2
    end
  end

  # returns whether the value matches a true value, such as "yes", "true", or "on".
  def to_boolean(value)
    [ "yes", "true", "on" ].include?(value.downcase)
  end

  def read_environment_variable
    # process the environment variable
    if e = ENV["GLARKOPTS"]
      options = e.split(/\s+/)
      while options.length > 0
        opt = options.shift
        process_option(opt, options)
      end
    end
  end

  # sets output a la grep
  def set_grep_output
    @highlight         = false
    @show_line_numbers = false
    @after             = 0
    @before            = 0
    @grep_output       = true
  end

  def read_options
    Log.log ""
    nargs = ARGV.size
    args = ARGV.dup

    @expr = nil

    while ARGV.length > 0
      arg = ARGV.shift
      break if @expr = process_option(arg, ARGV)
    end

    unless @expr
      # were any options processed?

      # A lone option of "-v" means version, if there was nothing else on the
      # command line. For grep compatibility, "-v" with an expression has to
      # mean an inverted match.

      if nargs == 1 && args[0] == "-v"
        show_version
      elsif nargs > 0
        $stderr.print "No expression provided.\n"
      end
      
      $stderr.print "Usage: glark [options] expression file...\n"
      $stderr.print "Try `glark --help' for more information.\n"
      exit 1
    end
  end

  def process_option(opt, args)
    Log.log "processing option #{opt}"
    case opt

      # after (context)
    when "-A"
      @after = args.shift.to_i
    when /^--after-context=(\d+)/
      @after = $1.to_i

      # before (context)
    when "-B"
      @before = args.shift.to_i
    when /^--before-context=(\d+)/
      @before = $1.to_i

      # after (range)
    when "-f", "--after"
      @range_start = args.shift

      # before (range)
    when "-b", "--before"
      @range_end = args.shift

      # range
    when "-R", "--range"
      @range_start, @range_end = args.shift, args.shift

      # context
    when "-C"
      nxt = args.shift
      # keep it if it is a number, else use the default
      if nxt =~ /^\d+/
        @before = @after = nxt.to_i
      else
        @before = @after = 2
        args.unshift(nxt)
      end
    when /^--context(=(\d+))?/
      @after = @before = if $2 then $2.to_i else 2 end
    when /^-(\d+)$/
      @after = @before = $1.to_i

      # highlighting
    when "-u", "--highlight"
      @highlight = true
    when "-U", "--no-highlight"
      @highlight = false
      
      # version
    when "-V", "--version"
      show_version

      # verbose
    when "-_"
      @verbose = 1
      Log.verbose = @verbose
    when /^--verbos(?:e|ity)(?:=(\d+))?/
      @verbose = $1 ? $1.to_i : 1
      Log.log "setting verbose to #{@verbose}"
      Log.verbose = @verbose

    when "-v", "--invert-match"
      @invert_match = true
      @exit_status  = 0
    when "-i", "--ignore-case"
      @nocase = true

      # grep
    when "-g", "--grep"
      set_grep_output

      # help
    when "-?", "--help"
      GlarkHelp.new
      exit 0

      # regexp explanation
    when "-e", "--explain"
      @explain = true

      # line numbers
    when "-N", "--no-line-number"
      @show_line_numbers = false
    when "-n", "--line-number"
      @show_line_numbers = true

      # quiet
    when "-q", "-s", "--quiet", "--messages"
      @quiet = true
    when "-Q", "-S", "--no-quiet", "--no-messages"
      @quiet = false

    when "-m", "--match-limit"
      @num_matches = args.shift.to_i
      
      # whole words
    when "-w", "--word", "--word-regexp"
      @whole_words = true
    when "-W", "--no-word", "--no-word-regexp"
      @whole_words = false

      # whole lines
    when "-x", "--line-regexp"
      @whole_lines = true
    when "-X", "--no-line-regexp"
      @whole_lines = false

      # file names only
    when "-l", "--files-with-matches"
      @file_names_only = true
      @invert_match = false
    when  "-L", "--files-without-match"
      @file_names_only = true
      @invert_match = true

      # colors
    when "-T", "--text-color"
      @text_highlight = make_highlight(opt, args.shift)
    when "-F", "--file-color"
      @file_highlight = make_highlight(opt, args.shift)

    when "-c", "--count"
      @count = true

    when "-Z", "--null"
      @write_null = true

    when "-M", "--exclude-matching-filenames"
      @exclude_matching_files = true
      
    when "-d"
      @directory = args.shift
    when /^--directories=(\w+)/
      @directory = $1

    when "-r", "--recurse"
      @directory = "recurse"

    when "-o", "-a"
      ec = ExpressionCreator.new(opt, args)
      @expr = ec.expr
      return @expr               # we are done.

    when "-h", /^--with-?filenames?$/
      @show_file_names = true
      
    when "-H", /^--no-?filenames?$/
      @show_file_names = false
      
    when /^--binary-files?=\"?(\w+)\"?/
      @binary_files = $1
      Log.log "set binary_files to #{@binary_files}"

    when "-y", "--extract-matches"
      Log.log "set extract matches"
      @extract_matches = true
      
    when /^(\-\w)(.+)/
      opt, rest = $1, "-" + $2
      puts "opt, rest = #{opt}, #{rest}" if @verbose
      args.unshift(rest)
      puts "args = #{args}" if @verbose
      return process_option(opt, args)
      
      # the expression
    else
      Log.log "not an option: #{opt}"
      if args
        ec = ExpressionCreator.new(opt, args)
        @expr = ec.expr
        return @expr            # we are done.
      end
    end
    return nil                  # we're not done.
  end

  # check options for collisions/data validity
  def validate
    if @range_start && @range_end
      pctre = Regexp.new(/([\.\d]+)%/)
      smd = pctre.match(@range_start)
      emd = pctre.match(@range_end)
      if !smd == !emd
        if smd
          if smd[1].to_f > emd[1].to_f
            puts "ERROR: range start (#{smd}) follows range end (#{emd})"
            exit 2
          end
        elsif @range_start.to_i > @range_end.to_i
          puts "ERROR: range start (#{@range_start}) follows range end (#{@range_end})"
          exit 2
        end
      end
    end
  end

  def show_version
    print @package, ", version ", @version, "\n"
    print "Written by Jeff Pace (jpace@glark.org).\n"
    print "Released under the Lesser GNU Public License.\n"
    exit 0
  end
  
end


if __FILE__ == $0
  Log.verbose = true
  gopt = GlarkOptions.new(ARGV)
  gopt.run
  methods = GlarkOptions.instance_methods
  methods.sort.each do |meth|
    # call the accessor for every setter method
    if meth[-1, 1] == "="
      acc = meth[0 .. -2]
      m = gopt.method(acc)
      puts "#{acc}: #{m.call}"
    end
  end
end
